// C Source File
// Created 9/28/2002; 2:05:24 PM

#define USE_TI89		// Compile for TI-89
#define USE_TI92PLUS		// Compile for TI-92 Plus
#define USE_V200		// Compile for V200

#define OPTIMIZE_ROM_CALLS	// Use ROM Call Optimization

#define MIN_AMS 100		// Compile for AMS 1.00 or higher

#define SAVE_SCREEN		// Save/Restore LCD Contents

#include <tigcclib.h>		// Include All Header Files
#include "lvlfind_.h"
#include "levmenu_.h"
#include "levstruct.h"
#include "make_file.h"

#define NO_SAVE_FILE_SUPPORT
#include "save.h"

char *gamefolder;

// Main Function
void
_main (void)
{
  char buffer[HOWMANY * MAXLEN];
  short map_count;
  short a, b;
  short temp_count;
  short number;
  unsigned short full_offset = 0;
  char map_count_str[4];
  char *allmaps = NULL;
  short exit = 0;
  unsigned char *filedata_in, *filedata_out;
  char filename[9];
  char buf[30];
  SYM_ENTRY *symptr_in = NULL, *symptr_out = NULL;
  HANDLE out;

  clrscr ();
  FontSetSys (F_6x8);
  printf ("Number of Maps?\n");
  InputNum (map_count_str, 3);
  map_count = atoi (map_count_str);
  if (!map_count)
    return;

  allmaps = (char *) malloc (map_count * MAXLEN);

  number = findAllFilesSet (buffer, "alev", HOWMANY);
  if (!number)
    return;

  printf ("\nLevel set name:\n");
  InputInfo (filename, 8);
  if (!filename[0])
    return;

  for (a = 0; a < map_count; a++)
    {
      number = findAllFilesSet (buffer, "alev", HOWMANY);
      ClrScr ();
      sprintf (buf, "Levelset #%i / %i", a + 1, map_count);
      DrawStr (25, 10, buf, A_NORMAL);
      number = domenu (35, 28, buffer, number, 0);
      if (number == -1)
	{
	  exit++;
	  break;
	}
      strcpy (allmaps + a * MAXLEN, buffer + number * MAXLEN);
    }

  if (checkarchived (filename))
    return;
//Now, parse all of the selected maps into a single level set.
  if (DerefSym (SymFind (SYMSTR (filename))))
    SymDel (SYMSTR (filename));
  out = CreateFile (filename);
  if (out != H_NULL)
    {
      filedata_out = HLock (out);
      filedata_out += 2;
    }
  else
    return;
  AppendCharToFile (out, 0);
//AppendCharToFile(out,0);


  number = 0;

  for (a = 0; a < map_count; a++)
    {
//Open the file the user chose earlier
      symptr_in = DerefSym (SymFind (SYMSTR (allmaps + a * MAXLEN)));
      if (symptr_in)
	{
	  filedata_in = HLock (symptr_in->handle);
	  filedata_in += 2;
	}
      else
	{
	  ST_helpMsg ("Error!");
	  CloseFile (out);
	  if (out)
	    HeapUnlock (out);
	  if (allmaps != NULL)
	    free (allmaps);
	  if (symptr_in)
	    HeapUnlock (symptr_in->handle);
	  if (symptr_out)
	    HeapUnlock (symptr_out->handle);
	  return;
	}
      number += *(short *) filedata_in;
    }



  for (a = 0; a < map_count; a++)
    {
//Open the file the user chose earlier
      symptr_in = DerefSym (SymFind (SYMSTR (allmaps + a * MAXLEN)));
      filedata_in = HLock (symptr_in->handle);
      filedata_in += 2;


      temp_count = *(short *) filedata_in;
//next comes a list of MAP_HEADERS
      MAP_HEADER *header = (MAP_HEADER *) (filedata_in + 2);

//Copy every level from the selected set to the new set
      for (b = 0; b < temp_count; b++)
	{
	  MAP_HEADER header_temp;
	  memcpy (&header_temp, header + b, sizeof (MAP_HEADER));
	  header_temp.offset = full_offset;
	  AppendBlockToFile (out, &header_temp, sizeof (MAP_HEADER));
	  full_offset += (header + b)->xsize * (header + b)->ysize;
	}



      HeapUnlock (symptr_in->handle);
    }

  for (a = 0; a < map_count; a++)
    {
//Open the file the user chose earlier
      symptr_in = DerefSym (SymFind (SYMSTR (allmaps + a * MAXLEN)));
      filedata_in = HLock (symptr_in->handle);
      filedata_in += 2;

      temp_count = *(short *) filedata_in;
//next comes a list of MAP_HEADERS
      MAP_HEADER *header = (MAP_HEADER *) (filedata_in + 2);
//next comes the actual tilemaps
      unsigned char *map_data =
	(unsigned char *) (filedata_in + 2 +
			   sizeof (MAP_HEADER) * temp_count);

//adding the offset for the ith header to the start of the tilemaps will get a pointer
//to the start of the tilemap we want
#define get_tile_map(i) (map_data + header[i].offset)

//Copy every level from the selected set to the new set

      for (b = 0; b < temp_count; b++)
	{
	  AppendBlockToFile (out, get_tile_map (b),
			     sizeof (char) * header[b].xsize *
			     header[b].ysize);
	}

      if (symptr_in)
	HeapUnlock (symptr_in->handle);
    }
  CloseFile (out);



  symptr_out = DerefSym (SymFind (SYMSTR (filename)));
  if (symptr_out)
    {
      filedata_out = HLock (symptr_out->handle);
      filedata_out += 2;
      *(short *) filedata_out = number;
    }




//*(short *)filedata_out = real_map_count;
  if (out)
    HeapUnlock (out);
  if (allmaps != NULL)
    free (allmaps);
  if (symptr_in)
    HeapUnlock (symptr_in->handle);
  if (symptr_out)
    HeapUnlock (symptr_out->handle);
}
