// C Header File
// Created 9/15/2002; 10:31:06 AM

#define AI_MOVE_AMT MOVE_SPEED
#define AWAY_AMT 30

void
AITurn (FATSPRITE * spr, short right, short increment)
{
  spr->orientation += (right) ? increment : -increment;
  // now clamp the camera orientation to the valid range of 0 ... 575
  spr->orientation = FAT_ClampAngle (spr->orientation);
}

void
AIMove (FATCONFIG * fc, FATSPRITE * spr, short stepx, short stepy,
	short newxpos, short newypos)
{
  short newxmod = newxpos >> 6;
  short newymod = newypos >> 6;

  //-------------------------------------------------------------------------
  // check if we are still within the map
  //-------------------------------------------------------------------------
  if (newxmod > fc->map_width - 1 ||
      newymod > fc->map_height - 1 || newxmod < 0 || newymod < 0)
    {
      return;
    }

  if (!ValidMove (fc, newxpos, newypos))
    {				// if new position is invalid ...
      // ...but old position is OK, we're against a wall to east or west
      if (ValidMove (fc, spr->xpos, newypos))
	{
	  // don't change angle if movement is perpendicular to the wall
	  if (stepy != 0)
	    {
	      // if sign(stepx)==sign(stepy), rotate clockwise = turn right
	      if ((stepx > 0) == (stepy > 0))
		AITurn (spr, 1, ANGLE_015 / 2);
	      else
		AITurn (spr, 0, ANGLE_015 / 2);
	      // still allow movement along y = parallel to wall
	      spr->ypos = newypos;
	    }
	}
      // ...but (new x, old y) is OK, we're against a wall to north or south
      if (ValidMove (fc, newxpos, spr->ypos))
	{
	  // don't change angle if movement is perpendicular to the wall
	  if (stepx != 0)
	    {
	      // if sign(stepx)==sign(stepy), rotate counterclockwise = turn left
	      if ((stepx > 0) == (stepy > 0))
		AITurn (spr, 0, ANGLE_015 / 2);
	      else
		AITurn (spr, 1, ANGLE_015 / 2);
	      // still allow movement along x = parallel to wall
	      spr->xpos = newxpos;
	    }
	}
      // ...otherwise we're in a corner, so do nothing
      return;
    }

  //-------------------------------------------------------------------------
  // everything okay -> let's set the new position
  //-------------------------------------------------------------------------
  spr->xpos = newxpos;
  spr->ypos = newypos;
}


void
SpawnPlayer (short player_idx, short model)
{
  FATSPRITE *cursprite = gamedata.player[player_idx].spr;
  //-------------------------------------------------------------------------
  // initialize sprite structure and setup its fields
  //-------------------------------------------------------------------------
  FAT_InitSprite (cursprite);
  cursprite->drawmode = SPRITEMODE_SIZE64;
  cursprite->nr_textures = 8;
  gamedata.player[player_idx].model = model;
  cursprite->textures = sprite_tex + gamedata.player[player_idx].model;
  cursprite->orientation = random (ANGLE_360 - 1) + 1;
  cursprite->xpos = (respawns[player_idx].x * 64) + 32;
  cursprite->ypos = (respawns[player_idx].y * 64) + 32;


}

void
SpawnAllPlayers (FATCONFIG * fc)
{
  short a;
  short model0 = SPRIDX_wolf11;	//Gray
  short model1 = SPRIDX_homer11;	//White

  for (a = gamedata.sprites.players; a >= 0; a--)
    {
      if (gamedata.gametype == DM)
	{
	  model0 = random (TOTAL_TEXCONFIGS_IN_a3dgfx2) & ~7;
	  model1 = random (TOTAL_TEXCONFIGS_IN_a3dgfx2) & ~7;
	}
      SpawnPlayer (a, (a % 2) ? model1 : model0);
    }

  fc->cam_xpos = player->xpos;
  fc->cam_ypos = player->ypos;
  fc->cam_orientation = player->orientation;
  player->drawmode = SPRITEMODE_UNUSED;
  player->nr_textures = 1;
  player->textures = sprite_tex;


  for (a = 0; a < MAX_PLAYERS + 1; a++)
    {
      gamedata.player[a].health = 100;
      gamedata.player[a].score = 0;
      switch (gamedata.gametype)
	{
	case DM:
	  gamedata.player[a].desire = ENEMY_DM_WALK;
	  gamedata.player[a].team = 0;
	  break;
	case TEAM:
	  gamedata.player[a].desire = ENEMY_TEAM_WALK;
	  gamedata.player[a].team = a % 2;
	  break;
	case CTF:
	  gamedata.player[a].desire = ENEMY_CTF_WALK;
	  gamedata.player[a].team = a % 2;
	  gamedata.player[a].order = (a - gamedata.player[a].team) / 2;
	  break;
	}
      gamedata.player[a].data = 0;
      gamedata.player[a].move = 0;
      gamedata.player[a].dir = 1;
      gamedata.player[a].rx = 0;
      gamedata.player[a].ry = 0;
      gamedata.player[a].sangle = 0;
      gamedata.player[a].rocket_idx = 0;
      gamedata.player[a].rocket_multiplier = 1;
      gamedata.player[a].multiplier_count = MULTIPLIER_DIVISOR;
    }

}

void
PlayersHandle (FATCONFIG * fc)
{
  short cur;

/*---------Rocket and rocket multiplier stuff---------*/
  if (gamedata.player[0].multiplier_count < MAX_GAME_CYCLES_FOR_MULTIPLIER)
    gamedata.player[0].multiplier_count++;

  gamedata.player[0].rocket_multiplier =
    gamedata.player[0].multiplier_count / MULTIPLIER_DIVISOR;


  for (cur = 1; cur < gamedata.sprites.players + 1; cur++)
    {
      if (gamedata.player[cur].multiplier_count <
	  MAX_GAME_CYCLES_FOR_MULTIPLIER)
	gamedata.player[cur].multiplier_count++;
      gamedata.player[cur].rocket_multiplier =
	gamedata.player[cur].multiplier_count / MULTIPLIER_DIVISOR;
/*-------------------------------------------------------*/

      //AIcall is defined in defs.h.
      //This runs something in ai.h, which contains all bot logic.
      AIcall (cur);

    }

}
